#!/usr/bin/env python
# -*- coding: utf-8 -*-

__author__ = "Ryan McGuire (ryan@enigmacurry.com)"
__date__   = "Mon Aug 18 14:26:56 2008"


"""Scrape a Simple Machines based forum calendar and convert to ical format"""

from datetime import datetime, timedelta
import sys
import re
import urllib2
import urlparse
import logging

from BeautifulSoup import BeautifulSoup
from ical import Calendar, Event

def month_sequence(start_date, end_date):
    "Generate a sequence of months and years within the date range"""
    month = start_date.month
    year  = start_date.year
    while True:
        yield((month,year))
        if month < 12:
            month += 1
        else:
            year += 1
            month = 1
        if datetime(year, month, 1) > end_date:
            break
        
class SMFParser:
    """Parses a Simple Machines Forum's calendar and exports to iCal format

    An optional filter can be applied that must match inside each event div
    in order to be created into an event:

    >>> p = SMFParser()
    >>> p.filter(r"Events: ")

    Each event div should have the phrase "Events: " in order to be matched.    
    """
    def __init__(self, name, forum_url, start_date, end_date):
        self.name = name
        self.forum_url = forum_url
        self.event_filters = set()
        self.parsed = None
        self.start_date = start_date
        self.end_date = end_date

    def parse(self):
        """Parse the calendar within the timeframe given.

        This tool parses whole months.
        If date range not given will do six month past and six months future"""

        cal = Calendar()
        cal.name = self.name
        cal.domain = urlparse.urlparse(self.forum_url)[1]
        
        for month,year in month_sequence(self.start_date, self.end_date):
            month_url = self.forum_url + "?action=calendar;year=%s;month=%s" % (year,month)
            logging.info("Downloading %s..." % month_url)
            data = urllib2.urlopen(month_url).read().decode("utf-8")
            soup = BeautifulSoup(data)
            for box in soup.find("div", id="bodyarea").findAll("td",{"class":re.compile("(windowbg)|(calendar_today)")}):
                #Make sure this is a date box
                try:
                    day = int(box.next.replace("\t","").replace("\n",""))
                except ValueError:
                    continue
                if not (0 < day <= 31):
                    continue
                for event_div in box.findChildren("div",{"class":"smalltext"}):
                    should_ignore = False
                    for filter in self.event_filters:
                        if re.search(filter, str(event_div)):
                            should_ignore = True
                            break
                    if should_ignore:
                        continue
                    for link in event_div.findAll("a"):
                        event = Event()
                        event.summary = link.next
                        event.description = link['href']
                        event.start = datetime(year, month, day)
                        event.all_day = True
                        cal.events.add(event)
        self.parsed = cal
        return cal
                    
    def save(self, filename):
        f = open(filename, "wb")
        if not self.parsed:
            self.parse()
        f.write(str(self.parsed))
        logging.info("File saved: " + filename)
        f.close()


if __name__ == '__main__':
    from optparse import OptionParser
    parser = OptionParser(version="%prog 0.1")
    parser.set_description(
        "Download a SMF 1.x forum calendar and dump in iCal format")
    parser.set_usage(
        "usage: %prog -u http://yourforum.com/index.php -o cal.ics")
    parser.add_option("-u","--url",dest="forum_url",
                      help="URL of forum (up to and including /index.php)",
                      metavar="URL")
    parser.add_option("-n","--name",dest="forum_name",
                      help="Name of the Forum / Calendar (name goes in .ics file)",
                      metavar="Name")
    parser.add_option("-i","--ignore-re",dest="ignore_re",
                      help="Ignore any event containing this regular expression (specify as many -i as you want)",
                      action="append",
                      metavar="EXPR")
    parser.add_option("-o","--output",dest="output_file",
                      help="iCal filename to write",
                      metavar="File")
    parser.add_option("-v","--verbose",dest="verbose",
                      action="store_true",default=False,
                      help="Be verbose about process")
    parser.add_option("--months-backward", dest="months_backward",
                      help="Number of months to go backward",
                      metavar="NUM", default="6")
    parser.add_option("--months-forward", dest="months_forward",
                      help="Number of months to go forward",
                      metavar="NUM", default="6")
    (options, args) = parser.parse_args()

    if None in (options.forum_url, options.output_file):
        parser.print_help()
        sys.exit(1)

    if options.verbose:
        logging.basicConfig(level=logging.INFO)

    if not options.forum_name:
        options.forum_name = urlparse.urlparse(options.forum_url)[1] + " calendar"

    days_forward = int(options.months_forward) * 31
    days_backward = int(options.months_backward) * 31

    parser = SMFParser(options.forum_name,options.forum_url,
                       (datetime.now() - timedelta(days_backward)),
                       (datetime.now() + timedelta(days_forward)))
    if options.ignore_re:
        for ignore_re in options.ignore_re:
            parser.event_filters.add(ignore_re)
    parser.save(options.output_file)
    
